@@ -16,8 +16,10 @@ import logging
 import os
 import queue
 import threading
+import time
 import traceback
 
+from ovs.db import custom_index
 from ovs.db import idl
 from ovs import poller
 
@@ -65,10 +67,10 @@ class Connection(object):
         """
         self.timeout = timeout
         self.txns = TransactionQueue(1)
-        self.lock = threading.Lock()
+        self.lock = threading.RLock()
         self.idl = idl
         self.thread = None
-        self._is_running = None
+        self.is_running = None
 
     def start(self):
         """Start the connection."""
@@ -83,43 +85,48 @@ class Connection(object):
                     # An ovs.db.Idl class has no post_connect
                     pass
             self.poller = poller.Poller()
-            self._is_running = True
+            self.is_running = True
             self.thread = threading.Thread(target=self.run)
             self.thread.setDaemon(True)
             self.thread.start()
 
     def run(self):
         errors = 0
-        while self._is_running:
+        while self.is_running:
             # If we fail in an Idl call, we could have missed an update
             # from the server, leaving us out of sync with ovsdb-server.
-            # It is not safe to continue without restarting the connection,
-            # though it is likely that the error is unrecoverable, so only try
-            # a few times before bailing completely.
+            # It is not safe to continue without restarting the connection.
+            # Though it is likely that the error is unrecoverable, keep trying
+            # indefinitely just in case.
             try:
                 self.idl.wait(self.poller)
                 self.poller.fd_wait(self.txns.alert_fileno, poller.POLLIN)
-                # TODO(jlibosva): Remove next line once losing connection to
-                #                 ovsdb is solved.
-                self.poller.timer_wait(self.timeout * 1000)
                 self.poller.block()
-                self.idl.run()
+                with self.lock:
+                    self.idl.run()
             except Exception as e:
                 # This shouldn't happen, but is possible if there is a bug
                 # in python-ovs
                 errors += 1
                 LOG.exception(e)
-                if errors <= 3:
+                with self.lock:
                     self.idl.force_reconnect()
-                    idlutils.wait_for_change(self.idl, self.timeout)
-                    continue
-                self._is_running = False
-                break
+                    try:
+                        idlutils.wait_for_change(self.idl, self.timeout)
+                    except Exception as e:
+                        # This could throw the same exception as idl.run()
+                        # or Exception("timeout"), either way continue
+                        LOG.exception(e)
+                sleep = min(2 ** errors, 60)
+                LOG.info("Trying to recover, sleeping %s seconds", sleep)
+                time.sleep(sleep)
+                continue
             errors = 0
             txn = self.txns.get_nowait()
             if txn is not None:
                 try:
-                    txn.results.put(txn.do_commit())
+                    with self.lock:
+                        txn.results.put(txn.do_commit())
                 except Exception as ex:
                     er = idlutils.ExceptionResult(ex=ex,
                                                   tb=traceback.format_exc())
@@ -127,9 +134,9 @@ class Connection(object):
                 self.txns.task_done()
 
     def stop(self, timeout=None):
-        if not self._is_running:
+        if not self.is_running:
             return True
-        self._is_running = False
+        self.is_running = False
         self.txns.put(None)
         self.thread.join(timeout)
         if self.thread.is_alive():
@@ -144,7 +151,8 @@ class Connection(object):
             self.txns.put(txn, timeout=self.timeout)
         except queue.Full:
             raise exceptions.TimeoutException(commands=txn.commands,
-                                              timeout=self.timeout)
+                                              timeout=self.timeout,
+                                              cause='TXN queue is full')
 
 
 class OvsdbIdl(idl.Idl):
@@ -161,3 +169,33 @@ class OvsdbIdl(idl.Idl):
         An example would be to set up Idl notification handling for watching
         and unwatching certain OVSDB change events
         """
+
+    def update_tables(self, tables, schema):
+        """Add the tables to the current Idl if they are present in the schema
+
+        :param tables: List of tables to be registered
+        :type tables: List
+        :param schema: Schema description
+        :type schema: dict or string
+        """
+
+        schema_helper = idlutils.create_schema_helper(schema)
+
+        # Register only available registered tables - DB downgrade, and the
+        # newly added tables - DB upgrade
+        for table in self.tables.keys() | tables:
+            if table in schema_helper.schema_json['tables']:
+                schema_helper.register_table(table)
+
+        schema = schema_helper.get_idl_schema()
+        self._db = schema
+        self.tables = schema.tables
+        for table in schema.tables.values():
+            for column in table.columns.values():
+                if not hasattr(column, 'alert'):
+                    column.alert = True
+            table.need_table = False
+            table.rows = custom_index.IndexedRows(table)
+            table.idl = self
+            table.condition = [True]
+            table.cond_changed = False
