@@ -310,6 +310,13 @@ class TestQoSOps(OvnNorthboundTest):
         self.assertRaises(TypeError, self.api.qos_del, self.switch.uuid,
                           priority=0)
 
+    def test_qos_del_ls_not_present_if_exists_true(self):
+        self.api.qos_del('some_other_ls').execute(check_error=True)
+
+    def test_qos_del_ls_not_present_if_exists_false(self):
+        cmd = self.api.qos_del('some_other_ls', if_exists=False)
+        self.assertRaises(RuntimeError, cmd.execute, check_error=True)
+
     def test_qos_list(self):
         r1 = self._qos_add('from-lport', 0, 'output == "fake_port"', 1000)
         r2 = self._qos_add('from-lport', 1, 'output == "fake_port2"', 1000)
@@ -1449,7 +1456,7 @@ class TestLsDnsOps(OvnNorthboundTest):
 
         ls1 = self._ls_add('ls1')
         self.api.ls_set_dns_records(ls1.uuid, [dns1.uuid, dns2.uuid]).execute()
-        self.assertItemsEqual([dns1.uuid, dns2.uuid],
+        self.assertCountEqual([dns1.uuid, dns2.uuid],
                               [dns.uuid for dns in ls1.dns_records])
 
         self.api.ls_clear_dns_records(ls1.uuid).execute()
@@ -1462,15 +1469,15 @@ class TestLsDnsOps(OvnNorthboundTest):
         ls1 = self._ls_add('ls1')
 
         self.api.ls_add_dns_record(ls1.uuid, dns1.uuid).execute()
-        self.assertItemsEqual([dns1.uuid],
+        self.assertCountEqual([dns1.uuid],
                               [dns.uuid for dns in ls1.dns_records])
 
         self.api.ls_add_dns_record(ls1.uuid, dns2.uuid).execute()
-        self.assertItemsEqual([dns1.uuid, dns2.uuid],
+        self.assertCountEqual([dns1.uuid, dns2.uuid],
                               [dns.uuid for dns in ls1.dns_records])
 
         self.api.ls_remove_dns_record(ls1.uuid, dns2.uuid).execute()
-        self.assertItemsEqual([dns1.uuid],
+        self.assertCountEqual([dns1.uuid],
                               [dns.uuid for dns in ls1.dns_records])
         self.api.ls_remove_dns_record(ls1.uuid, dns1.uuid).execute()
         self.assertEqual([], ls1.dns_records)
